# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

from pydantic.v1 import validate_arguments, ValidationError
from typing_extensions import Annotated
from typing import overload, Optional, Union, Awaitable

from pydantic.v1 import Field, StrictBool, StrictInt, StrictStr, conint

from typing import Optional

from linebot.v3.audience.models.add_audience_to_audience_group_request import AddAudienceToAudienceGroupRequest
from linebot.v3.audience.models.audience_group_create_route import AudienceGroupCreateRoute
from linebot.v3.audience.models.audience_group_status import AudienceGroupStatus
from linebot.v3.audience.models.create_audience_group_request import CreateAudienceGroupRequest
from linebot.v3.audience.models.create_audience_group_response import CreateAudienceGroupResponse
from linebot.v3.audience.models.create_click_based_audience_group_request import CreateClickBasedAudienceGroupRequest
from linebot.v3.audience.models.create_click_based_audience_group_response import CreateClickBasedAudienceGroupResponse
from linebot.v3.audience.models.create_imp_based_audience_group_request import CreateImpBasedAudienceGroupRequest
from linebot.v3.audience.models.create_imp_based_audience_group_response import CreateImpBasedAudienceGroupResponse
from linebot.v3.audience.models.get_audience_data_response import GetAudienceDataResponse
from linebot.v3.audience.models.get_audience_groups_response import GetAudienceGroupsResponse
from linebot.v3.audience.models.get_shared_audience_data_response import GetSharedAudienceDataResponse
from linebot.v3.audience.models.get_shared_audience_groups_response import GetSharedAudienceGroupsResponse
from linebot.v3.audience.models.update_audience_group_description_request import UpdateAudienceGroupDescriptionRequest

from linebot.v3.audience.async_api_client import AsyncApiClient
from linebot.v3.audience.api_response import ApiResponse
from linebot.v3.audience.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class AsyncManageAudience(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = AsyncApiClient.get_default()
        self.api_client = api_client
        self.line_base_path = "https://api.line.me"


    @overload
    async def add_audience_to_audience_group(self, add_audience_to_audience_group_request : AddAudienceToAudienceGroupRequest, **kwargs) -> None:  # noqa: E501
        ...

    @overload
    def add_audience_to_audience_group(self, add_audience_to_audience_group_request : AddAudienceToAudienceGroupRequest, async_req: Optional[bool]=True, **kwargs) -> None:  # noqa: E501
        ...

    @validate_arguments
    def add_audience_to_audience_group(self, add_audience_to_audience_group_request : AddAudienceToAudienceGroupRequest, async_req: Optional[bool]=None, **kwargs) -> Union[None, Awaitable[None]]:  # noqa: E501
        """add_audience_to_audience_group  # noqa: E501

        Add user IDs or Identifiers for Advertisers (IFAs) to an audience for uploading user IDs (by JSON)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.add_audience_to_audience_group(add_audience_to_audience_group_request, async_req=True)
        >>> result = thread.get()

        :param add_audience_to_audience_group_request: (required)
        :type add_audience_to_audience_group_request: AddAudienceToAudienceGroupRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the add_audience_to_audience_group_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.add_audience_to_audience_group_with_http_info(add_audience_to_audience_group_request, **kwargs)  # noqa: E501

    @validate_arguments
    def add_audience_to_audience_group_with_http_info(self, add_audience_to_audience_group_request : AddAudienceToAudienceGroupRequest, **kwargs) -> ApiResponse:  # noqa: E501
        """add_audience_to_audience_group  # noqa: E501

        Add user IDs or Identifiers for Advertisers (IFAs) to an audience for uploading user IDs (by JSON)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.add_audience_to_audience_group_with_http_info(add_audience_to_audience_group_request, async_req=True)
        >>> result = thread.get()

        :param add_audience_to_audience_group_request: (required)
        :type add_audience_to_audience_group_request: AddAudienceToAudienceGroupRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'add_audience_to_audience_group_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_audience_to_audience_group" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['add_audience_to_audience_group_request'] is not None:
            _body_params = _params['add_audience_to_audience_group_request']

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/upload', 'PUT',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def create_audience_group(self, create_audience_group_request : CreateAudienceGroupRequest, **kwargs) -> CreateAudienceGroupResponse:  # noqa: E501
        ...

    @overload
    def create_audience_group(self, create_audience_group_request : CreateAudienceGroupRequest, async_req: Optional[bool]=True, **kwargs) -> CreateAudienceGroupResponse:  # noqa: E501
        ...

    @validate_arguments
    def create_audience_group(self, create_audience_group_request : CreateAudienceGroupRequest, async_req: Optional[bool]=None, **kwargs) -> Union[CreateAudienceGroupResponse, Awaitable[CreateAudienceGroupResponse]]:  # noqa: E501
        """create_audience_group  # noqa: E501

        Create audience for uploading user IDs (by JSON)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_audience_group(create_audience_group_request, async_req=True)
        >>> result = thread.get()

        :param create_audience_group_request: (required)
        :type create_audience_group_request: CreateAudienceGroupRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CreateAudienceGroupResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the create_audience_group_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.create_audience_group_with_http_info(create_audience_group_request, **kwargs)  # noqa: E501

    @validate_arguments
    def create_audience_group_with_http_info(self, create_audience_group_request : CreateAudienceGroupRequest, **kwargs) -> ApiResponse:  # noqa: E501
        """create_audience_group  # noqa: E501

        Create audience for uploading user IDs (by JSON)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_audience_group_with_http_info(create_audience_group_request, async_req=True)
        >>> result = thread.get()

        :param create_audience_group_request: (required)
        :type create_audience_group_request: CreateAudienceGroupRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CreateAudienceGroupResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'create_audience_group_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_audience_group" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['create_audience_group_request'] is not None:
            _body_params = _params['create_audience_group_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '202': "CreateAudienceGroupResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/upload', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def create_click_based_audience_group(self, create_click_based_audience_group_request : CreateClickBasedAudienceGroupRequest, **kwargs) -> CreateClickBasedAudienceGroupResponse:  # noqa: E501
        ...

    @overload
    def create_click_based_audience_group(self, create_click_based_audience_group_request : CreateClickBasedAudienceGroupRequest, async_req: Optional[bool]=True, **kwargs) -> CreateClickBasedAudienceGroupResponse:  # noqa: E501
        ...

    @validate_arguments
    def create_click_based_audience_group(self, create_click_based_audience_group_request : CreateClickBasedAudienceGroupRequest, async_req: Optional[bool]=None, **kwargs) -> Union[CreateClickBasedAudienceGroupResponse, Awaitable[CreateClickBasedAudienceGroupResponse]]:  # noqa: E501
        """create_click_based_audience_group  # noqa: E501

        Create audience for click-based retargeting  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_click_based_audience_group(create_click_based_audience_group_request, async_req=True)
        >>> result = thread.get()

        :param create_click_based_audience_group_request: (required)
        :type create_click_based_audience_group_request: CreateClickBasedAudienceGroupRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CreateClickBasedAudienceGroupResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the create_click_based_audience_group_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.create_click_based_audience_group_with_http_info(create_click_based_audience_group_request, **kwargs)  # noqa: E501

    @validate_arguments
    def create_click_based_audience_group_with_http_info(self, create_click_based_audience_group_request : CreateClickBasedAudienceGroupRequest, **kwargs) -> ApiResponse:  # noqa: E501
        """create_click_based_audience_group  # noqa: E501

        Create audience for click-based retargeting  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_click_based_audience_group_with_http_info(create_click_based_audience_group_request, async_req=True)
        >>> result = thread.get()

        :param create_click_based_audience_group_request: (required)
        :type create_click_based_audience_group_request: CreateClickBasedAudienceGroupRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CreateClickBasedAudienceGroupResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'create_click_based_audience_group_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_click_based_audience_group" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['create_click_based_audience_group_request'] is not None:
            _body_params = _params['create_click_based_audience_group_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '202': "CreateClickBasedAudienceGroupResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/click', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def create_imp_based_audience_group(self, create_imp_based_audience_group_request : CreateImpBasedAudienceGroupRequest, **kwargs) -> CreateImpBasedAudienceGroupResponse:  # noqa: E501
        ...

    @overload
    def create_imp_based_audience_group(self, create_imp_based_audience_group_request : CreateImpBasedAudienceGroupRequest, async_req: Optional[bool]=True, **kwargs) -> CreateImpBasedAudienceGroupResponse:  # noqa: E501
        ...

    @validate_arguments
    def create_imp_based_audience_group(self, create_imp_based_audience_group_request : CreateImpBasedAudienceGroupRequest, async_req: Optional[bool]=None, **kwargs) -> Union[CreateImpBasedAudienceGroupResponse, Awaitable[CreateImpBasedAudienceGroupResponse]]:  # noqa: E501
        """create_imp_based_audience_group  # noqa: E501

        Create audience for impression-based retargeting  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_imp_based_audience_group(create_imp_based_audience_group_request, async_req=True)
        >>> result = thread.get()

        :param create_imp_based_audience_group_request: (required)
        :type create_imp_based_audience_group_request: CreateImpBasedAudienceGroupRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CreateImpBasedAudienceGroupResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the create_imp_based_audience_group_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.create_imp_based_audience_group_with_http_info(create_imp_based_audience_group_request, **kwargs)  # noqa: E501

    @validate_arguments
    def create_imp_based_audience_group_with_http_info(self, create_imp_based_audience_group_request : CreateImpBasedAudienceGroupRequest, **kwargs) -> ApiResponse:  # noqa: E501
        """create_imp_based_audience_group  # noqa: E501

        Create audience for impression-based retargeting  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_imp_based_audience_group_with_http_info(create_imp_based_audience_group_request, async_req=True)
        >>> result = thread.get()

        :param create_imp_based_audience_group_request: (required)
        :type create_imp_based_audience_group_request: CreateImpBasedAudienceGroupRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CreateImpBasedAudienceGroupResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'create_imp_based_audience_group_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_imp_based_audience_group" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['create_imp_based_audience_group_request'] is not None:
            _body_params = _params['create_imp_based_audience_group_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '202': "CreateImpBasedAudienceGroupResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/imp', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def delete_audience_group(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], **kwargs) -> None:  # noqa: E501
        ...

    @overload
    def delete_audience_group(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], async_req: Optional[bool]=True, **kwargs) -> None:  # noqa: E501
        ...

    @validate_arguments
    def delete_audience_group(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], async_req: Optional[bool]=None, **kwargs) -> Union[None, Awaitable[None]]:  # noqa: E501
        """delete_audience_group  # noqa: E501

        Delete audience  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_audience_group(audience_group_id, async_req=True)
        >>> result = thread.get()

        :param audience_group_id: The audience ID. (required)
        :type audience_group_id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the delete_audience_group_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.delete_audience_group_with_http_info(audience_group_id, **kwargs)  # noqa: E501

    @validate_arguments
    def delete_audience_group_with_http_info(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], **kwargs) -> ApiResponse:  # noqa: E501
        """delete_audience_group  # noqa: E501

        Delete audience  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_audience_group_with_http_info(audience_group_id, async_req=True)
        >>> result = thread.get()

        :param audience_group_id: The audience ID. (required)
        :type audience_group_id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'audience_group_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_audience_group" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['audience_group_id']:
            _path_params['audienceGroupId'] = _params['audience_group_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/{audienceGroupId}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def get_audience_data(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], **kwargs) -> GetAudienceDataResponse:  # noqa: E501
        ...

    @overload
    def get_audience_data(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], async_req: Optional[bool]=True, **kwargs) -> GetAudienceDataResponse:  # noqa: E501
        ...

    @validate_arguments
    def get_audience_data(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], async_req: Optional[bool]=None, **kwargs) -> Union[GetAudienceDataResponse, Awaitable[GetAudienceDataResponse]]:  # noqa: E501
        """get_audience_data  # noqa: E501

        Gets audience data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_audience_data(audience_group_id, async_req=True)
        >>> result = thread.get()

        :param audience_group_id: The audience ID. (required)
        :type audience_group_id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetAudienceDataResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_audience_data_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.get_audience_data_with_http_info(audience_group_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_audience_data_with_http_info(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], **kwargs) -> ApiResponse:  # noqa: E501
        """get_audience_data  # noqa: E501

        Gets audience data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_audience_data_with_http_info(audience_group_id, async_req=True)
        >>> result = thread.get()

        :param audience_group_id: The audience ID. (required)
        :type audience_group_id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetAudienceDataResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'audience_group_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_audience_data" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['audience_group_id']:
            _path_params['audienceGroupId'] = _params['audience_group_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetAudienceDataResponse",
            '400': "ErrorResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/{audienceGroupId}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def get_audience_groups(self, page : Annotated[conint(strict=True, ge=1), Field(..., description="The page to return when getting (paginated) results. Must be 1 or higher.")], description : Annotated[Optional[StrictStr], Field(description="The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. ")] = None, status : Annotated[Optional[AudienceGroupStatus], Field(description="The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. ")] = None, size : Annotated[Optional[conint(strict=True, le=40)], Field(description="The number of audiences per page. Default: 20 Max: 40 ")] = None, includes_external_public_groups : Annotated[Optional[StrictBool], Field(description="true (default): Get public audiences created in all channels linked to the same bot. false: Get audiences created in the same channel. ")] = None, create_route : Annotated[Optional[AudienceGroupCreateRoute], Field(description="How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. ")] = None, **kwargs) -> GetAudienceGroupsResponse:  # noqa: E501
        ...

    @overload
    def get_audience_groups(self, page : Annotated[conint(strict=True, ge=1), Field(..., description="The page to return when getting (paginated) results. Must be 1 or higher.")], description : Annotated[Optional[StrictStr], Field(description="The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. ")] = None, status : Annotated[Optional[AudienceGroupStatus], Field(description="The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. ")] = None, size : Annotated[Optional[conint(strict=True, le=40)], Field(description="The number of audiences per page. Default: 20 Max: 40 ")] = None, includes_external_public_groups : Annotated[Optional[StrictBool], Field(description="true (default): Get public audiences created in all channels linked to the same bot. false: Get audiences created in the same channel. ")] = None, create_route : Annotated[Optional[AudienceGroupCreateRoute], Field(description="How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. ")] = None, async_req: Optional[bool]=True, **kwargs) -> GetAudienceGroupsResponse:  # noqa: E501
        ...

    @validate_arguments
    def get_audience_groups(self, page : Annotated[conint(strict=True, ge=1), Field(..., description="The page to return when getting (paginated) results. Must be 1 or higher.")], description : Annotated[Optional[StrictStr], Field(description="The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. ")] = None, status : Annotated[Optional[AudienceGroupStatus], Field(description="The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. ")] = None, size : Annotated[Optional[conint(strict=True, le=40)], Field(description="The number of audiences per page. Default: 20 Max: 40 ")] = None, includes_external_public_groups : Annotated[Optional[StrictBool], Field(description="true (default): Get public audiences created in all channels linked to the same bot. false: Get audiences created in the same channel. ")] = None, create_route : Annotated[Optional[AudienceGroupCreateRoute], Field(description="How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. ")] = None, async_req: Optional[bool]=None, **kwargs) -> Union[GetAudienceGroupsResponse, Awaitable[GetAudienceGroupsResponse]]:  # noqa: E501
        """get_audience_groups  # noqa: E501

        Gets data for more than one audience.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_audience_groups(page, description, status, size, includes_external_public_groups, create_route, async_req=True)
        >>> result = thread.get()

        :param page: The page to return when getting (paginated) results. Must be 1 or higher. (required)
        :type page: int
        :param description: The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. 
        :type description: str
        :param status: The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. 
        :type status: AudienceGroupStatus
        :param size: The number of audiences per page. Default: 20 Max: 40 
        :type size: int
        :param includes_external_public_groups: true (default): Get public audiences created in all channels linked to the same bot. false: Get audiences created in the same channel. 
        :type includes_external_public_groups: bool
        :param create_route: How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. 
        :type create_route: AudienceGroupCreateRoute
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetAudienceGroupsResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_audience_groups_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.get_audience_groups_with_http_info(page, description, status, size, includes_external_public_groups, create_route, **kwargs)  # noqa: E501

    @validate_arguments
    def get_audience_groups_with_http_info(self, page : Annotated[conint(strict=True, ge=1), Field(..., description="The page to return when getting (paginated) results. Must be 1 or higher.")], description : Annotated[Optional[StrictStr], Field(description="The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. ")] = None, status : Annotated[Optional[AudienceGroupStatus], Field(description="The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. ")] = None, size : Annotated[Optional[conint(strict=True, le=40)], Field(description="The number of audiences per page. Default: 20 Max: 40 ")] = None, includes_external_public_groups : Annotated[Optional[StrictBool], Field(description="true (default): Get public audiences created in all channels linked to the same bot. false: Get audiences created in the same channel. ")] = None, create_route : Annotated[Optional[AudienceGroupCreateRoute], Field(description="How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. ")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """get_audience_groups  # noqa: E501

        Gets data for more than one audience.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_audience_groups_with_http_info(page, description, status, size, includes_external_public_groups, create_route, async_req=True)
        >>> result = thread.get()

        :param page: The page to return when getting (paginated) results. Must be 1 or higher. (required)
        :type page: int
        :param description: The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. 
        :type description: str
        :param status: The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. 
        :type status: AudienceGroupStatus
        :param size: The number of audiences per page. Default: 20 Max: 40 
        :type size: int
        :param includes_external_public_groups: true (default): Get public audiences created in all channels linked to the same bot. false: Get audiences created in the same channel. 
        :type includes_external_public_groups: bool
        :param create_route: How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. 
        :type create_route: AudienceGroupCreateRoute
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetAudienceGroupsResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'page',
            'description',
            'status',
            'size',
            'includes_external_public_groups',
            'create_route'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_audience_groups" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('page') is not None:  # noqa: E501
            _query_params.append(('page', _params['page']))

        if _params.get('description') is not None:  # noqa: E501
            _query_params.append(('description', _params['description']))

        if _params.get('status') is not None:  # noqa: E501
            _query_params.append(('status', _params['status'].value))

        if _params.get('size') is not None:  # noqa: E501
            _query_params.append(('size', _params['size']))

        if _params.get('includes_external_public_groups') is not None:  # noqa: E501
            _query_params.append(('includesExternalPublicGroups', _params['includes_external_public_groups']))

        if _params.get('create_route') is not None:  # noqa: E501
            _query_params.append(('createRoute', _params['create_route'].value))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetAudienceGroupsResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/list', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def get_shared_audience_data(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], **kwargs) -> GetSharedAudienceDataResponse:  # noqa: E501
        ...

    @overload
    def get_shared_audience_data(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], async_req: Optional[bool]=True, **kwargs) -> GetSharedAudienceDataResponse:  # noqa: E501
        ...

    @validate_arguments
    def get_shared_audience_data(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], async_req: Optional[bool]=None, **kwargs) -> Union[GetSharedAudienceDataResponse, Awaitable[GetSharedAudienceDataResponse]]:  # noqa: E501
        """get_shared_audience_data  # noqa: E501

        Gets audience data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_shared_audience_data(audience_group_id, async_req=True)
        >>> result = thread.get()

        :param audience_group_id: The audience ID. (required)
        :type audience_group_id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetSharedAudienceDataResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_shared_audience_data_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.get_shared_audience_data_with_http_info(audience_group_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_shared_audience_data_with_http_info(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], **kwargs) -> ApiResponse:  # noqa: E501
        """get_shared_audience_data  # noqa: E501

        Gets audience data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_shared_audience_data_with_http_info(audience_group_id, async_req=True)
        >>> result = thread.get()

        :param audience_group_id: The audience ID. (required)
        :type audience_group_id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetSharedAudienceDataResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'audience_group_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_shared_audience_data" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['audience_group_id']:
            _path_params['audienceGroupId'] = _params['audience_group_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetSharedAudienceDataResponse",
            '400': "ErrorResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/shared/{audienceGroupId}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def get_shared_audience_groups(self, page : Annotated[conint(strict=True, ge=1), Field(..., description="The page to return when getting (paginated) results. Must be 1 or higher.")], description : Annotated[Optional[StrictStr], Field(description="The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. ")] = None, status : Annotated[Optional[AudienceGroupStatus], Field(description="The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. ")] = None, size : Annotated[Optional[conint(strict=True, le=40)], Field(description="The number of audiences per page. Default: 20 Max: 40 ")] = None, create_route : Annotated[Optional[AudienceGroupCreateRoute], Field(description="How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. ")] = None, includes_owned_audience_groups : Annotated[Optional[StrictBool], Field(description="true: Include audienceGroups owned by LINE Official Account Manager false: Respond only audienceGroups shared by Business Manager ")] = None, **kwargs) -> GetSharedAudienceGroupsResponse:  # noqa: E501
        ...

    @overload
    def get_shared_audience_groups(self, page : Annotated[conint(strict=True, ge=1), Field(..., description="The page to return when getting (paginated) results. Must be 1 or higher.")], description : Annotated[Optional[StrictStr], Field(description="The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. ")] = None, status : Annotated[Optional[AudienceGroupStatus], Field(description="The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. ")] = None, size : Annotated[Optional[conint(strict=True, le=40)], Field(description="The number of audiences per page. Default: 20 Max: 40 ")] = None, create_route : Annotated[Optional[AudienceGroupCreateRoute], Field(description="How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. ")] = None, includes_owned_audience_groups : Annotated[Optional[StrictBool], Field(description="true: Include audienceGroups owned by LINE Official Account Manager false: Respond only audienceGroups shared by Business Manager ")] = None, async_req: Optional[bool]=True, **kwargs) -> GetSharedAudienceGroupsResponse:  # noqa: E501
        ...

    @validate_arguments
    def get_shared_audience_groups(self, page : Annotated[conint(strict=True, ge=1), Field(..., description="The page to return when getting (paginated) results. Must be 1 or higher.")], description : Annotated[Optional[StrictStr], Field(description="The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. ")] = None, status : Annotated[Optional[AudienceGroupStatus], Field(description="The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. ")] = None, size : Annotated[Optional[conint(strict=True, le=40)], Field(description="The number of audiences per page. Default: 20 Max: 40 ")] = None, create_route : Annotated[Optional[AudienceGroupCreateRoute], Field(description="How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. ")] = None, includes_owned_audience_groups : Annotated[Optional[StrictBool], Field(description="true: Include audienceGroups owned by LINE Official Account Manager false: Respond only audienceGroups shared by Business Manager ")] = None, async_req: Optional[bool]=None, **kwargs) -> Union[GetSharedAudienceGroupsResponse, Awaitable[GetSharedAudienceGroupsResponse]]:  # noqa: E501
        """get_shared_audience_groups  # noqa: E501

        Gets data for more than one audience, including those shared by the Business Manager.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_shared_audience_groups(page, description, status, size, create_route, includes_owned_audience_groups, async_req=True)
        >>> result = thread.get()

        :param page: The page to return when getting (paginated) results. Must be 1 or higher. (required)
        :type page: int
        :param description: The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. 
        :type description: str
        :param status: The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. 
        :type status: AudienceGroupStatus
        :param size: The number of audiences per page. Default: 20 Max: 40 
        :type size: int
        :param create_route: How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. 
        :type create_route: AudienceGroupCreateRoute
        :param includes_owned_audience_groups: true: Include audienceGroups owned by LINE Official Account Manager false: Respond only audienceGroups shared by Business Manager 
        :type includes_owned_audience_groups: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetSharedAudienceGroupsResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_shared_audience_groups_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.get_shared_audience_groups_with_http_info(page, description, status, size, create_route, includes_owned_audience_groups, **kwargs)  # noqa: E501

    @validate_arguments
    def get_shared_audience_groups_with_http_info(self, page : Annotated[conint(strict=True, ge=1), Field(..., description="The page to return when getting (paginated) results. Must be 1 or higher.")], description : Annotated[Optional[StrictStr], Field(description="The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. ")] = None, status : Annotated[Optional[AudienceGroupStatus], Field(description="The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. ")] = None, size : Annotated[Optional[conint(strict=True, le=40)], Field(description="The number of audiences per page. Default: 20 Max: 40 ")] = None, create_route : Annotated[Optional[AudienceGroupCreateRoute], Field(description="How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. ")] = None, includes_owned_audience_groups : Annotated[Optional[StrictBool], Field(description="true: Include audienceGroups owned by LINE Official Account Manager false: Respond only audienceGroups shared by Business Manager ")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """get_shared_audience_groups  # noqa: E501

        Gets data for more than one audience, including those shared by the Business Manager.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_shared_audience_groups_with_http_info(page, description, status, size, create_route, includes_owned_audience_groups, async_req=True)
        >>> result = thread.get()

        :param page: The page to return when getting (paginated) results. Must be 1 or higher. (required)
        :type page: int
        :param description: The name of the audience(s) to return. You can search for partial matches. This is case-insensitive, meaning AUDIENCE and audience are considered identical. If omitted, the name of the audience(s) will not be used as a search criterion. 
        :type description: str
        :param status: The status of the audience(s) to return. If omitted, the status of the audience(s) will not be used as a search criterion. 
        :type status: AudienceGroupStatus
        :param size: The number of audiences per page. Default: 20 Max: 40 
        :type size: int
        :param create_route: How the audience was created. If omitted, all audiences are included.  `OA_MANAGER`: Return only audiences created with LINE Official Account Manager (opens new window). `MESSAGING_API`: Return only audiences created with Messaging API. 
        :type create_route: AudienceGroupCreateRoute
        :param includes_owned_audience_groups: true: Include audienceGroups owned by LINE Official Account Manager false: Respond only audienceGroups shared by Business Manager 
        :type includes_owned_audience_groups: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetSharedAudienceGroupsResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'page',
            'description',
            'status',
            'size',
            'create_route',
            'includes_owned_audience_groups'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_shared_audience_groups" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('page') is not None:  # noqa: E501
            _query_params.append(('page', _params['page']))

        if _params.get('description') is not None:  # noqa: E501
            _query_params.append(('description', _params['description']))

        if _params.get('status') is not None:  # noqa: E501
            _query_params.append(('status', _params['status'].value))

        if _params.get('size') is not None:  # noqa: E501
            _query_params.append(('size', _params['size']))

        if _params.get('create_route') is not None:  # noqa: E501
            _query_params.append(('createRoute', _params['create_route'].value))

        if _params.get('includes_owned_audience_groups') is not None:  # noqa: E501
            _query_params.append(('includesOwnedAudienceGroups', _params['includes_owned_audience_groups']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetSharedAudienceGroupsResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/shared/list', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def update_audience_group_description(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], update_audience_group_description_request : UpdateAudienceGroupDescriptionRequest, **kwargs) -> None:  # noqa: E501
        ...

    @overload
    def update_audience_group_description(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], update_audience_group_description_request : UpdateAudienceGroupDescriptionRequest, async_req: Optional[bool]=True, **kwargs) -> None:  # noqa: E501
        ...

    @validate_arguments
    def update_audience_group_description(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], update_audience_group_description_request : UpdateAudienceGroupDescriptionRequest, async_req: Optional[bool]=None, **kwargs) -> Union[None, Awaitable[None]]:  # noqa: E501
        """update_audience_group_description  # noqa: E501

        Renames an existing audience.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_audience_group_description(audience_group_id, update_audience_group_description_request, async_req=True)
        >>> result = thread.get()

        :param audience_group_id: The audience ID. (required)
        :type audience_group_id: int
        :param update_audience_group_description_request: (required)
        :type update_audience_group_description_request: UpdateAudienceGroupDescriptionRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the update_audience_group_description_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.update_audience_group_description_with_http_info(audience_group_id, update_audience_group_description_request, **kwargs)  # noqa: E501

    @validate_arguments
    def update_audience_group_description_with_http_info(self, audience_group_id : Annotated[StrictInt, Field(..., description="The audience ID.")], update_audience_group_description_request : UpdateAudienceGroupDescriptionRequest, **kwargs) -> ApiResponse:  # noqa: E501
        """update_audience_group_description  # noqa: E501

        Renames an existing audience.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_audience_group_description_with_http_info(audience_group_id, update_audience_group_description_request, async_req=True)
        >>> result = thread.get()

        :param audience_group_id: The audience ID. (required)
        :type audience_group_id: int
        :param update_audience_group_description_request: (required)
        :type update_audience_group_description_request: UpdateAudienceGroupDescriptionRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'audience_group_id',
            'update_audience_group_description_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_audience_group_description" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['audience_group_id']:
            _path_params['audienceGroupId'] = _params['audience_group_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['update_audience_group_description_request'] is not None:
            _body_params = _params['update_audience_group_description_request']

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v2/bot/audienceGroup/{audienceGroupId}/updateDescription', 'PUT',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import os
import unittest
from builtins import open
import inspect

from linebot.v3 import (
    SignatureValidator, WebhookParser, WebhookHandler
)
from linebot.v3.webhooks.models import (
    MessageEvent, FollowEvent, UnfollowEvent, JoinEvent,
    LeaveEvent, PostbackEvent, BeaconEvent, AccountLinkEvent,
    MemberJoinedEvent, MemberLeftEvent,
    VideoPlayCompleteEvent, VideoPlayComplete, UnsendEvent, UnsendDetail,
    TextMessageContent, ImageMessageContent, VideoMessageContent, AudioMessageContent,
    LocationMessageContent, StickerMessageContent, FileMessageContent,
    UserSource, RoomSource, GroupSource,
    DeliveryContext
)
from linebot.v3.models import (
    UnknownEvent,
)
from linebot.v3.utils import PY3


class TestSignatureValidator(unittest.TestCase):
    def test_validate(self):
        signature_validator = SignatureValidator('channel_secret')

        self.assertEqual(
            signature_validator.validate(
                'bodybodybodybody', '/gg9a+LvFevTH1sd7XCQycD7tsWclCsInj7MhBHxN7k='),
            True
        )
        self.assertEqual(
            signature_validator.validate(
                'bodybodybodybody', 'invalid_signature'),
            False
        )


class TestWebhookParser(unittest.TestCase):
    def setUp(self):
        parser = WebhookParser('channel_secret')
        # mock
        parser.signature_validator.validate = lambda a, b: True
        self.parser = parser

    def test_parse(self):
        file_dir = os.path.dirname(__file__)
        webhook_sample_json_path = os.path.join(file_dir, 'text', 'webhook.json')
        with open(webhook_sample_json_path) as fp:
            body = fp.read()

        events = self.parser.parse(body, 'channel_secret')

        # events count
        self.assertEqual(len(events), 27)

        # MessageEvent, UserSource, TextMessageContent
        self.assertIsInstance(events[0], MessageEvent)
        self.assertEqual(events[0].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[0].type, 'message')
        self.assertEqual(events[0].mode, 'active')
        self.assertEqual(events[0].timestamp, 1462629479859)
        self.assertIsInstance(events[0].source, UserSource)
        self.assertEqual(events[0].source.type, 'user')
        self.assertEqual(events[0].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[0].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[0].delivery_context, DeliveryContext)
        self.assertFalse(events[0].delivery_context.is_redelivery)
        self.assertIsInstance(events[0].message, TextMessageContent)
        self.assertEqual(events[0].message.id, '325708')
        self.assertEqual(events[0].message.type, 'text')
        self.assertEqual(events[0].message.text, 'Hello, world')

        # MessageEvent, RoomSource, ImageMessageContent
        self.assertIsInstance(events[1], MessageEvent)
        self.assertEqual(events[1].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[1].type, 'message')
        self.assertEqual(events[1].mode, 'active')
        self.assertEqual(events[1].timestamp, 1462629479859)
        self.assertIsInstance(events[1].source, RoomSource)
        self.assertEqual(events[1].source.type, 'room')
        self.assertEqual(events[1].source.room_id, 'Ra8dbf4673c4c812cd491258042226c99')
        self.assertEqual(events[1].source.user_id, None)
        self.assertEqual(events[1].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[1].delivery_context, DeliveryContext)
        self.assertFalse(events[1].delivery_context.is_redelivery)
        self.assertIsInstance(events[1].message, ImageMessageContent)
        self.assertEqual(events[1].message.id, '325708')
        self.assertEqual(events[1].message.type, 'image')
        self.assertEqual(events[1].message.content_provider.type, 'external')
        self.assertEqual(events[1].message.content_provider.original_content_url,
                         "https://example.com")
        self.assertEqual(events[1].message.content_provider.preview_image_url,
                         "https://example.com")

        # MessageEvent, UserSource, VideoMessageContent
        self.assertIsInstance(events[2], MessageEvent)
        self.assertEqual(events[2].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[2].type, 'message')
        self.assertEqual(events[2].mode, 'active')
        self.assertEqual(events[2].timestamp, 1462629479859)
        self.assertIsInstance(events[2].source, UserSource)
        self.assertEqual(events[2].source.type, 'user')
        self.assertEqual(events[2].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[2].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[2].delivery_context, DeliveryContext)
        self.assertFalse(events[2].delivery_context.is_redelivery)
        self.assertIsInstance(events[2].message, VideoMessageContent)
        self.assertEqual(events[2].message.id, '325708')
        self.assertEqual(events[2].message.type, 'video')
        self.assertEqual(events[2].message.duration, 60000)
        self.assertEqual(events[2].message.content_provider.type, 'external')
        self.assertEqual(events[2].message.content_provider.original_content_url,
                         "https://example.com")
        self.assertEqual(events[2].message.content_provider.preview_image_url,
                         "https://example.com")

        # MessageEvent, UserSource, AudioMessageContent
        self.assertIsInstance(events[3], MessageEvent)
        self.assertEqual(events[3].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[3].type, 'message')
        self.assertEqual(events[3].mode, 'active')
        self.assertEqual(events[3].timestamp, 1462629479859)
        self.assertIsInstance(events[3].source, UserSource)
        self.assertEqual(events[3].source.type, 'user')
        self.assertEqual(events[3].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[3].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[3].delivery_context, DeliveryContext)
        self.assertFalse(events[3].delivery_context.is_redelivery)
        self.assertIsInstance(events[3].message, AudioMessageContent)
        self.assertEqual(events[3].message.id, '325708')
        self.assertEqual(events[3].message.type, 'audio')
        self.assertEqual(events[3].message.duration, 60000)
        self.assertEqual(events[3].message.content_provider.type, 'external')
        self.assertEqual(events[3].message.content_provider.original_content_url,
                         "https://example.com")

        # MessageEvent, UserSource, LocationMessageContent
        self.assertIsInstance(events[4], MessageEvent)
        self.assertEqual(events[4].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[4].type, 'message')
        self.assertEqual(events[4].mode, 'active')
        self.assertEqual(events[4].timestamp, 1462629479859)
        self.assertIsInstance(events[4].source, UserSource)
        self.assertEqual(events[4].source.type, 'user')
        self.assertEqual(events[4].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[4].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[4].delivery_context, DeliveryContext)
        self.assertFalse(events[4].delivery_context.is_redelivery)
        self.assertIsInstance(events[4].message, LocationMessageContent)
        self.assertEqual(events[4].message.id, '325708')
        self.assertEqual(events[4].message.type, 'location')
        self.assertEqual(events[4].message.title, 'my location')
        self.assertEqual(events[4].message.address, 'Tokyo')
        self.assertEqual(events[4].message.latitude, 35.65910807942215)
        self.assertEqual(events[4].message.longitude, 139.70372892916203)

        # MessageEvent, UserSource, StickerMessageContent
        self.assertIsInstance(events[5], MessageEvent)
        self.assertEqual(events[5].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[5].type, 'message')
        self.assertEqual(events[5].mode, 'active')
        self.assertEqual(events[5].timestamp, 1462629479859)
        self.assertIsInstance(events[5].source, UserSource)
        self.assertEqual(events[5].source.type, 'user')
        self.assertEqual(events[5].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[5].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[5].delivery_context, DeliveryContext)
        self.assertFalse(events[5].delivery_context.is_redelivery)
        self.assertIsInstance(events[5].message, StickerMessageContent)
        self.assertEqual(events[5].message.id, '325708')
        self.assertEqual(events[5].message.type, 'sticker')
        self.assertEqual(events[5].message.package_id, '1')
        self.assertEqual(events[5].message.sticker_id, '1')
        self.assertEqual(events[5].message.sticker_resource_type, 'STATIC')
        self.assertEqual(events[5].message.keywords[0], 'Love You')
        self.assertEqual(events[5].message.keywords[1], 'Love')
        self.assertEqual(events[5].message.text, 'Just sticker')

        # FollowEvent, User
        self.assertIsInstance(events[6], FollowEvent)
        self.assertEqual(events[6].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[6].type, 'follow')
        self.assertEqual(events[6].mode, 'active')
        self.assertEqual(events[6].timestamp, 1462629479859)
        self.assertIsInstance(events[6].source, UserSource)
        self.assertEqual(events[6].source.type, 'user')
        self.assertEqual(events[6].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[6].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[6].delivery_context, DeliveryContext)
        self.assertFalse(events[6].delivery_context.is_redelivery)

        # UnfollowEvent, User
        self.assertIsInstance(events[7], UnfollowEvent)
        self.assertEqual(events[7].type, 'unfollow')
        self.assertEqual(events[7].mode, 'active')
        self.assertEqual(events[7].timestamp, 1462629479859)
        self.assertIsInstance(events[7].source, UserSource)
        self.assertEqual(events[7].source.type, 'user')
        self.assertEqual(events[7].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[7].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[7].delivery_context, DeliveryContext)
        self.assertFalse(events[7].delivery_context.is_redelivery)

        # JoinEvent, GroupSource
        self.assertIsInstance(events[8], JoinEvent)
        self.assertEqual(events[8].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[8].type, 'join')
        self.assertEqual(events[8].mode, 'active')
        self.assertEqual(events[8].timestamp, 1462629479859)
        self.assertIsInstance(events[8].source, GroupSource)
        self.assertEqual(events[8].source.type, 'group')
        self.assertEqual(events[8].source.group_id, 'Ca56f94637cc4347f90a25382909b24b9')
        self.assertEqual(events[8].source.user_id, None)
        self.assertEqual(events[8].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[8].delivery_context, DeliveryContext)
        self.assertFalse(events[8].delivery_context.is_redelivery)

        # LeaveEvent, GroupSource
        self.assertIsInstance(events[9], LeaveEvent)
        self.assertEqual(events[9].type, 'leave')
        self.assertEqual(events[9].mode, 'active')
        self.assertEqual(events[9].timestamp, 1462629479859)
        self.assertIsInstance(events[9].source, GroupSource)
        self.assertEqual(events[9].source.type, 'group')
        self.assertEqual(events[9].source.group_id, 'Ca56f94637cc4347f90a25382909b24b9')
        self.assertEqual(events[9].source.user_id, None)
        self.assertEqual(events[9].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[9].delivery_context, DeliveryContext)
        self.assertFalse(events[9].delivery_context.is_redelivery)

        # PostbackEvent, UserSource
        self.assertIsInstance(events[10], PostbackEvent)
        self.assertEqual(events[10].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[10].type, 'postback')
        self.assertEqual(events[10].mode, 'active')
        self.assertEqual(events[10].timestamp, 1462629479859)
        self.assertIsInstance(events[10].source, UserSource)
        self.assertEqual(events[10].source.type, 'user')
        self.assertEqual(events[10].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[10].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[10].delivery_context, DeliveryContext)
        self.assertFalse(events[10].delivery_context.is_redelivery)
        self.assertEqual(events[10].postback.data, 'action=buyItem&itemId=123123&color=red')
        self.assertEqual(events[10].postback.params, {'datetime': '2017-12-25T01:00'})

        # BeaconEvent, UserSource
        self.assertIsInstance(events[11], BeaconEvent)
        self.assertEqual(events[11].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[11].type, 'beacon')
        self.assertEqual(events[11].mode, 'active')
        self.assertEqual(events[11].timestamp, 1462629479859)
        self.assertIsInstance(events[11].source, UserSource)
        self.assertEqual(events[11].source.type, 'user')
        self.assertEqual(events[11].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[11].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[11].delivery_context, DeliveryContext)
        self.assertFalse(events[11].delivery_context.is_redelivery)
        self.assertEqual(events[11].beacon.hwid, 'd41d8cd98f')
        self.assertEqual(events[11].beacon.type, 'enter')
        self.assertEqual(events[11].beacon.dm, None)

        # BeaconEvent, UserSource (with device message)
        self.assertIsInstance(events[12], BeaconEvent)
        self.assertEqual(events[12].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[12].type, 'beacon')
        self.assertEqual(events[12].mode, 'active')
        self.assertEqual(events[12].timestamp, 1462629479859)
        self.assertIsInstance(events[12].source, UserSource)
        self.assertEqual(events[12].source.type, 'user')
        self.assertEqual(events[12].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[12].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[12].delivery_context, DeliveryContext)
        self.assertFalse(events[12].delivery_context.is_redelivery)
        self.assertEqual(events[12].beacon.hwid, 'd41d8cd98f')
        self.assertEqual(events[12].beacon.type, 'enter')
        self.assertEqual(events[12].beacon.dm, '1234567890abcdef')

        # AccountEvent, UserSource
        self.assertIsInstance(events[13], AccountLinkEvent)
        self.assertEqual(events[13].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[13].type, 'accountLink')
        self.assertEqual(events[13].mode, 'active')
        self.assertEqual(events[13].timestamp, 1462629479859)
        self.assertIsInstance(events[13].source, UserSource)
        self.assertEqual(events[13].source.type, 'user')
        self.assertEqual(events[13].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[13].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[13].delivery_context, DeliveryContext)
        self.assertFalse(events[13].delivery_context.is_redelivery)
        self.assertEqual(events[13].link.result, 'ok')
        self.assertEqual(events[13].link.nonce, 'Vb771wDYtXuammLszK6h9A')

        # MessageEvent, GroupSource with userId, TextMessageContent
        self.assertIsInstance(events[14], MessageEvent)
        self.assertEqual(events[14].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[14].type, 'message')
        self.assertEqual(events[14].mode, 'active')
        self.assertEqual(events[14].timestamp, 1462629479859)
        self.assertIsInstance(events[14].source, GroupSource)
        self.assertEqual(events[14].source.type, 'group')
        self.assertEqual(events[14].source.group_id, 'Ca56f94637cc4347f90a25382909b24b9')
        self.assertEqual(events[14].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[14].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[14].delivery_context, DeliveryContext)
        self.assertFalse(events[14].delivery_context.is_redelivery)
        self.assertIsInstance(events[14].message, TextMessageContent)
        self.assertEqual(events[14].message.id, '325708')
        self.assertEqual(events[14].message.type, 'text')
        self.assertEqual(events[14].message.text, 'Hello, world')

        # MessageEvent, RoomSource with userId, TextMessageContent
        self.assertIsInstance(events[15], MessageEvent)
        self.assertEqual(events[15].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[15].type, 'message')
        self.assertEqual(events[15].mode, 'active')
        self.assertEqual(events[15].timestamp, 1462629479859)
        self.assertIsInstance(events[15].source, RoomSource)
        self.assertEqual(events[15].source.type, 'room')
        self.assertEqual(events[15].source.room_id, 'Ra8dbf4673c4c812cd491258042226c99')
        self.assertEqual(events[15].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[15].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[15].delivery_context, DeliveryContext)
        self.assertFalse(events[15].delivery_context.is_redelivery)
        self.assertIsInstance(events[15].message, TextMessageContent)
        self.assertEqual(events[15].message.id, '325708')
        self.assertEqual(events[15].message.type, 'text')
        self.assertEqual(events[15].message.text, 'Hello, world')

        # PostbackEvent, UserSource, with date params
        self.assertIsInstance(events[16], PostbackEvent)
        self.assertEqual(events[16].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[16].type, 'postback')
        self.assertEqual(events[16].mode, 'active')
        self.assertEqual(events[16].timestamp, 1462629479859)
        self.assertIsInstance(events[16].source, UserSource)
        self.assertEqual(events[16].source.type, 'user')
        self.assertEqual(events[16].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[16].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[16].delivery_context, DeliveryContext)
        self.assertFalse(events[16].delivery_context.is_redelivery)
        self.assertEqual(events[16].postback.data, 'action=buyItem&itemId=123123&color=red')
        self.assertEqual(events[16].postback.params['date'], '2013-04-01')

        # PostbackEvent, UserSource, with date params
        self.assertIsInstance(events[17], PostbackEvent)
        self.assertEqual(events[17].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[17].type, 'postback')
        self.assertEqual(events[17].mode, 'active')
        self.assertEqual(events[17].timestamp, 1462629479859)
        self.assertIsInstance(events[17].source, UserSource)
        self.assertEqual(events[17].source.type, 'user')
        self.assertEqual(events[17].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[17].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[17].delivery_context, DeliveryContext)
        self.assertFalse(events[17].delivery_context.is_redelivery)
        self.assertEqual(events[17].postback.data, 'action=buyItem&itemId=123123&color=red')
        self.assertEqual(events[17].postback.params['time'], '10:00')

        # PostbackEvent, UserSource, with date params
        self.assertIsInstance(events[18], PostbackEvent)
        self.assertEqual(events[18].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[18].type, 'postback')
        self.assertEqual(events[18].mode, 'active')
        self.assertEqual(events[18].timestamp, 1462629479859)
        self.assertIsInstance(events[18].source, UserSource)
        self.assertEqual(events[18].source.type, 'user')
        self.assertEqual(events[18].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[18].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[18].delivery_context, DeliveryContext)
        self.assertFalse(events[18].delivery_context.is_redelivery)
        self.assertEqual(events[18].postback.data, 'action=buyItem&itemId=123123&color=red')
        self.assertEqual(events[18].postback.params['datetime'], '2013-04-01T10:00')

        # TODO Fix range
        # MemberJoinedEvent
        self.assertIsInstance(events[19], MemberJoinedEvent)
        self.assertEqual(events[19].reply_token, '0f3779fba3b349968c5d07db31eabf65')
        self.assertEqual(events[19].type, 'memberJoined')
        self.assertEqual(events[19].mode, 'active')
        self.assertEqual(events[19].timestamp, 1462629479859)
        self.assertIsInstance(events[19].source, GroupSource)
        self.assertEqual(events[19].source.type, 'group')
        self.assertEqual(events[19].source.group_id, 'C4af4980629...')
        self.assertEqual(events[19].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[19].delivery_context, DeliveryContext)
        self.assertFalse(events[19].delivery_context.is_redelivery)
        self.assertEqual(len(events[19].joined.members), 2)
        self.assertIsInstance(events[19].joined.members[0], UserSource)
        self.assertEqual(events[19].joined.members[0].user_id, 'U4af4980629...')
        self.assertEqual(events[19].joined.members[1].user_id, 'U91eeaf62d9...')

        # MemberLeftEvent
        self.assertIsInstance(events[20], MemberLeftEvent)
        self.assertEqual(events[20].type, 'memberLeft')
        self.assertEqual(events[20].mode, 'active')
        self.assertEqual(events[20].timestamp, 1462629479960)
        self.assertIsInstance(events[20].source, GroupSource)
        self.assertEqual(events[20].source.type, 'group')
        self.assertEqual(events[20].source.group_id, 'C4af4980629...')
        self.assertEqual(events[20].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[20].delivery_context, DeliveryContext)
        self.assertFalse(events[20].delivery_context.is_redelivery)
        self.assertEqual(len(events[20].left.members), 2)
        self.assertIsInstance(events[20].left.members[0], UserSource)
        self.assertEqual(events[20].left.members[0].user_id, 'U4af4980629...')
        self.assertEqual(events[20].left.members[1].user_id, 'U91eeaf62d9...')

        # MessageEvent, UserSource, UserSource
        self.assertIsInstance(events[21], MessageEvent)
        self.assertEqual(events[21].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[21].type, 'message')
        self.assertEqual(events[21].mode, 'active')
        self.assertEqual(events[21].timestamp, 1462629479859)
        self.assertIsInstance(events[21].source, UserSource)
        self.assertEqual(events[21].source.type, 'user')
        self.assertEqual(events[21].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[21].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[21].delivery_context, DeliveryContext)
        self.assertFalse(events[21].delivery_context.is_redelivery)
        self.assertIsInstance(events[21].message, FileMessageContent)
        self.assertEqual(events[21].message.id, '325708')
        self.assertEqual(events[21].message.type, 'file')
        self.assertEqual(events[21].message.file_name, "file.txt")
        self.assertEqual(events[21].message.file_size, 2138)

        # UnsendEvent
        self.assertIsInstance(events[22], UnsendEvent)
        self.assertEqual(events[22].type, 'unsend')
        self.assertEqual(events[22].mode, 'active')
        self.assertEqual(events[22].timestamp, 1547817848122)
        self.assertIsInstance(events[22].source, GroupSource)
        self.assertEqual(events[22].source.type, 'group')
        self.assertEqual(events[22].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[22].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[22].delivery_context, DeliveryContext)
        self.assertFalse(events[22].delivery_context.is_redelivery)
        self.assertIsInstance(events[22].unsend, UnsendDetail)
        self.assertEqual(events[22].unsend.message_id, '325708')

        # VideoPlayCompleteEvent
        self.assertIsInstance(events[23], VideoPlayCompleteEvent)
        self.assertEqual(events[23].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[23].type, 'videoPlayComplete')
        self.assertEqual(events[23].mode, 'active')
        self.assertEqual(events[23].timestamp, 1462629479859)
        self.assertIsInstance(events[23].source, UserSource)
        self.assertEqual(events[23].source.type, 'user')
        self.assertEqual(events[23].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[23].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[23].delivery_context, DeliveryContext)
        self.assertFalse(events[23].delivery_context.is_redelivery)
        self.assertIsInstance(events[23].video_play_complete, VideoPlayComplete)
        self.assertEqual(events[23].video_play_complete.tracking_id, 'track_id')

        # MessageEvent, UserSource, ImageMessage with ImageSet
        self.assertIsInstance(events[1], MessageEvent)
        self.assertEqual(events[24].reply_token, 'fbf94e269485410da6b7e3a5e33283e8')
        self.assertEqual(events[24].type, 'message')
        self.assertEqual(events[24].mode, 'active')
        self.assertEqual(events[24].timestamp, 1627356924722)
        self.assertIsInstance(events[24].source, UserSource)
        self.assertEqual(events[24].source.type, 'user')
        self.assertEqual(events[24].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[24].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[24].delivery_context, DeliveryContext)
        self.assertFalse(events[24].delivery_context.is_redelivery)
        self.assertIsInstance(events[24].message, ImageMessageContent)
        self.assertEqual(events[24].message.id, '354718705033693861')
        self.assertEqual(events[24].message.type, 'image')
        self.assertEqual(events[24].message.content_provider.type, 'line')
        self.assertEqual(events[24].message.image_set.id, 'E005D41A7288F41B655')
        self.assertEqual(events[24].message.image_set.index, 2)
        self.assertEqual(events[24].message.image_set.total, 2)

        # MessageEvent, UserSource, TextMessage (Redeliveried)
        self.assertIsInstance(events[25], MessageEvent)
        self.assertEqual(events[25].reply_token, 'nHuyWiB7yP5Zw52FIkcQobQuGDXCTA')
        self.assertEqual(events[25].type, 'message')
        self.assertEqual(events[25].mode, 'active')
        self.assertEqual(events[25].timestamp, 1462629479859)
        self.assertIsInstance(events[25].source, UserSource)
        self.assertEqual(events[25].source.type, 'user')
        self.assertEqual(events[25].source.user_id, 'U206d25c2ea6bd87c17655609a1c37cb8')
        self.assertEqual(events[25].webhook_event_id, 'testwebhookeventid')
        self.assertIsInstance(events[25].delivery_context, DeliveryContext)
        self.assertTrue(events[25].delivery_context.is_redelivery)
        self.assertIsInstance(events[25].message, TextMessageContent)
        self.assertEqual(events[25].message.id, '325708')
        self.assertEqual(events[25].message.type, 'text')
        self.assertEqual(events[25].message.text, 'Hello, world')

        # UnknownEvent
        self.assertIsInstance(events[26], UnknownEvent)

        # TODO: richmenu switch

    def test_parse_webhook_req_without_destination(self):
        body = """
        {
            "events": [
                {
                    "replyToken": "00000000000000000000000000000000",
                    "type": "message",
                    "mode": "active",
                    "timestamp": 1561099010135,
                    "source": {
                        "type": "user",
                        "userId": "Udeadbeefdeadbeefdeadbeefdeadbeef"
                    },
                    "message": {
                        "id": "100001",
                        "type": "text",
                        "text": "Hello, world"
                    },
                    "webhookEventId": "testwebhookeventid",
                    "deliveryContext": {
                        "isRedelivery": false
                    }
                },
                {
                    "replyToken": "ffffffffffffffffffffffffffffffff",
                    "type": "message",
                    "mode": "active",
                    "timestamp": 1561099010135,
                    "source": {
                        "type": "user",
                        "userId": "Udeadbeefdeadbeefdeadbeefdeadbeef"
                    },
                    "message": {
                        "id": "100002",
                        "type": "sticker",
                        "packageId": "1",
                        "stickerId": "1",
                        "stickerResourceType": "ANIMATION"
                    },
                    "webhookEventId": "testwebhookeventid",
                    "deliveryContext": {
                        "isRedelivery": false
                    }
                }
            ]
        }
        """
        payload = self.parser.parse(body=body, signature='channel_secret', as_payload=True)
        self.assertEqual(None, payload.destination)


class TestWebhookHandler(unittest.TestCase):
    def setUp(self):
        self.handler = WebhookHandler('channel_secret')

        @self.handler.add(MessageEvent, message=TextMessageContent)
        def message_text(event, destination):
            self.assertEqual('message', event.type)
            self.assertEqual('text', event.message.type)
            self.assertEqual('U123', destination)

        @self.handler.add(MessageEvent,
                          message=(ImageMessageContent,
                                   VideoMessageContent,
                                   AudioMessageContent))
        def message_content(event):
            self.assertEqual('message', event.type)
            self.assertIn(
                event.message.type,
                ['image', 'video', 'audio']
            )

        @self.handler.add(MessageEvent, message=StickerMessageContent)
        def message_sticker(event):
            self.assertEqual('message', event.type)
            self.assertEqual('sticker', event.message.type)

        @self.handler.add(MessageEvent)
        def message(event):
            self.assertEqual('message', event.type)
            self.assertNotIn(
                event.message.type,
                ['text', 'image', 'video', 'audio', 'sticker']
            )

        @self.handler.add(FollowEvent)
        def follow(event, destination):
            self.assertEqual('follow', event.type)
            self.assertEqual('U123', destination)

        @self.handler.add(JoinEvent)
        def join(event):
            self.assertEqual('join', event.type)

        @self.handler.add(PostbackEvent)
        def postback(event):
            self.assertEqual('postback', event.type)

        @self.handler.add(BeaconEvent)
        def beacon(event):
            self.assertEqual('beacon', event.type)

        @self.handler.add(AccountLinkEvent)
        def account_link(event):
            self.assertEqual('accountLink', event.type)

        @self.handler.default()
        def default(event):
            self.assertNotIn(
                event.type,
                ['message', 'follow', 'join', 'postback', 'beacon', 'accountLink']
            )

    def test_handler(self):
        file_dir = os.path.dirname(__file__)
        webhook_sample_json_path = os.path.join(file_dir, 'text', 'webhook.json')
        with open(webhook_sample_json_path) as fp:
            body = fp.read()

        # mock
        self.handler.parser.signature_validator.validate = lambda a, b: True

        self.handler.handle(body, 'signature')


class TestInvokeWebhookHandler(unittest.TestCase):
    def setUp(self):
        def wrap(func):
            def wrapper(*args):
                if PY3:
                    arg_spec = inspect.getfullargspec(func)
                else:
                    arg_spec = inspect.getargspec(func)
                return func(*args[0:len(arg_spec.args)])

            return wrapper

        def func_with_0_args():
            assert True

        def func_with_1_arg(arg):
            assert arg

        def func_with_2_args(arg1, arg2):
            assert arg1 and arg2

        def func_with_1_arg_with_default(arg=False):
            assert arg

        def func_with_2_args_with_default(arg1=False, arg2=False):
            assert arg1 and arg2

        def func_with_1_arg_and_1_arg_with_default(arg1, arg2=False):
            assert arg1 and arg2

        @wrap
        def wrapped_func_with_0_args():
            assert True

        @wrap
        def wrapped_func_with_1_arg(arg):
            assert arg

        @wrap
        def wrapped_func_with_2_args(arg1, arg2):
            assert arg1 and arg2

        @wrap
        def wrapped_func_with_1_arg_with_default(arg=False):
            assert arg

        @wrap
        def wrapped_func_with_2_args_with_default(arg1=False, arg2=False):
            assert arg1 and arg2

        @wrap
        def wrapped_func_with_1_arg_and_1_arg_with_default(
                arg1, arg2=False):
            assert arg1 and arg2

        self.functions = [
            func_with_0_args,
            func_with_1_arg,
            func_with_2_args,
            func_with_1_arg_with_default,
            func_with_2_args_with_default,
            func_with_1_arg_and_1_arg_with_default,
            wrapped_func_with_0_args,
            wrapped_func_with_1_arg,
            wrapped_func_with_2_args,
            wrapped_func_with_1_arg_with_default,
            wrapped_func_with_2_args_with_default,
            wrapped_func_with_1_arg_and_1_arg_with_default,
        ]

    def test_invoke_func(self):
        class PayloadMock(object):
            def __init__(self):
                self.destination = True

        event = True
        payload = PayloadMock()

        for func in self.functions:
            WebhookHandler._WebhookHandler__invoke_func(
                func, event, payload
            )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, validator
from linebot.v3.messaging.models.acquisition_condition_response import AcquisitionConditionResponse
from linebot.v3.messaging.models.coupon_reward_response import CouponRewardResponse

class CouponResponse(BaseModel):
    """
    Detailed information about a coupon, including all properties and current status.
    """
    acquisition_condition: Optional[AcquisitionConditionResponse] = Field(None, alias="acquisitionCondition")
    barcode_image_url: Optional[StrictStr] = Field(None, alias="barcodeImageUrl", description="URL of the barcode image associated with the coupon. Used for in-store redemption.")
    coupon_code: Optional[StrictStr] = Field(None, alias="couponCode", description="Unique code to be presented by the user to redeem the coupon.")
    description: Optional[StrictStr] = Field(None, description="Detailed description of the coupon. Displayed to users.")
    end_timestamp: Optional[StrictInt] = Field(None, alias="endTimestamp", description="Coupon expiration time (epoch seconds). Coupon cannot be used after this time.")
    image_url: Optional[StrictStr] = Field(None, alias="imageUrl", description="URL of the main image representing the coupon. Displayed in the coupon list.")
    max_acquire_count: Optional[StrictInt] = Field(None, alias="maxAcquireCount", description="Maximum number of coupons that can be issued in total.")
    max_use_count_per_ticket: Optional[StrictInt] = Field(None, alias="maxUseCountPerTicket", description="Maximum number of times a single coupon ticket can be used.")
    max_ticket_per_user: Optional[StrictInt] = Field(None, alias="maxTicketPerUser", description="Maximum number of coupon tickets a single user can acquire.")
    start_timestamp: Optional[StrictInt] = Field(None, alias="startTimestamp", description="Coupon start time (epoch seconds). Coupon can be used from this time.")
    title: Optional[StrictStr] = Field(None, description="Title of the coupon. Displayed in the coupon list.")
    usage_condition: Optional[StrictStr] = Field(None, alias="usageCondition", description="Conditions for using the coupon. Shown to users.")
    reward: Optional[CouponRewardResponse] = None
    visibility: Optional[StrictStr] = Field(None, description="Visibility of the coupon. Determines who can see or acquire the coupon.")
    timezone: Optional[StrictStr] = Field(None, description="Timezone for interpreting start and end timestamps.")
    coupon_id: Optional[StrictStr] = Field(None, alias="couponId", description="Unique identifier of the coupon.")
    created_timestamp: Optional[StrictInt] = Field(None, alias="createdTimestamp", description="Created timestamp (seconds) of the coupon.")
    status: Optional[StrictStr] = Field(None, description="Current status of the coupon.")

    __properties = ["acquisitionCondition", "barcodeImageUrl", "couponCode", "description", "endTimestamp", "imageUrl", "maxAcquireCount", "maxUseCountPerTicket", "maxTicketPerUser", "startTimestamp", "title", "usageCondition", "reward", "visibility", "timezone", "couponId", "createdTimestamp", "status"]

    @validator('visibility')
    def visibility_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('UNLISTED', 'PUBLIC', 'PRIVATE'):
            raise ValueError("must be one of enum values ('UNLISTED', 'PUBLIC', 'PRIVATE')")
        return value

    @validator('timezone')
    def timezone_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('ETC_GMT_MINUS_12', 'ETC_GMT_MINUS_11', 'PACIFIC_HONOLULU', 'AMERICA_ANCHORAGE', 'AMERICA_LOS_ANGELES', 'AMERICA_PHOENIX', 'AMERICA_CHICAGO', 'AMERICA_NEW_YORK', 'AMERICA_CARACAS', 'AMERICA_SANTIAGO', 'AMERICA_ST_JOHNS', 'AMERICA_SAO_PAULO', 'ETC_GMT_MINUS_2', 'ATLANTIC_CAPE_VERDE', 'EUROPE_LONDON', 'EUROPE_PARIS', 'EUROPE_ISTANBUL', 'EUROPE_MOSCOW', 'ASIA_TEHRAN', 'ASIA_TBILISI', 'ASIA_KABUL', 'ASIA_TASHKENT', 'ASIA_COLOMBO', 'ASIA_KATHMANDU', 'ASIA_ALMATY', 'ASIA_RANGOON', 'ASIA_BANGKOK', 'ASIA_TAIPEI', 'ASIA_TOKYO', 'AUSTRALIA_DARWIN', 'AUSTRALIA_SYDNEY', 'ASIA_VLADIVOSTOK', 'ETC_GMT_PLUS_12', 'PACIFIC_TONGATAPU'):
            raise ValueError("must be one of enum values ('ETC_GMT_MINUS_12', 'ETC_GMT_MINUS_11', 'PACIFIC_HONOLULU', 'AMERICA_ANCHORAGE', 'AMERICA_LOS_ANGELES', 'AMERICA_PHOENIX', 'AMERICA_CHICAGO', 'AMERICA_NEW_YORK', 'AMERICA_CARACAS', 'AMERICA_SANTIAGO', 'AMERICA_ST_JOHNS', 'AMERICA_SAO_PAULO', 'ETC_GMT_MINUS_2', 'ATLANTIC_CAPE_VERDE', 'EUROPE_LONDON', 'EUROPE_PARIS', 'EUROPE_ISTANBUL', 'EUROPE_MOSCOW', 'ASIA_TEHRAN', 'ASIA_TBILISI', 'ASIA_KABUL', 'ASIA_TASHKENT', 'ASIA_COLOMBO', 'ASIA_KATHMANDU', 'ASIA_ALMATY', 'ASIA_RANGOON', 'ASIA_BANGKOK', 'ASIA_TAIPEI', 'ASIA_TOKYO', 'AUSTRALIA_DARWIN', 'AUSTRALIA_SYDNEY', 'ASIA_VLADIVOSTOK', 'ETC_GMT_PLUS_12', 'PACIFIC_TONGATAPU')")
        return value

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('DRAFT', 'RUNNING', 'CLOSED'):
            raise ValueError("must be one of enum values ('DRAFT', 'RUNNING', 'CLOSED')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CouponResponse:
        """Create an instance of CouponResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of acquisition_condition
        if self.acquisition_condition:
            _dict['acquisitionCondition'] = self.acquisition_condition.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of reward
        if self.reward:
            _dict['reward'] = self.reward.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CouponResponse:
        """Create an instance of CouponResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CouponResponse.parse_obj(obj)

        _obj = CouponResponse.parse_obj({
            "acquisition_condition": AcquisitionConditionResponse.from_dict(obj.get("acquisitionCondition")) if obj.get("acquisitionCondition") is not None else None,
            "barcode_image_url": obj.get("barcodeImageUrl"),
            "coupon_code": obj.get("couponCode"),
            "description": obj.get("description"),
            "end_timestamp": obj.get("endTimestamp"),
            "image_url": obj.get("imageUrl"),
            "max_acquire_count": obj.get("maxAcquireCount"),
            "max_use_count_per_ticket": obj.get("maxUseCountPerTicket"),
            "max_ticket_per_user": obj.get("maxTicketPerUser"),
            "start_timestamp": obj.get("startTimestamp"),
            "title": obj.get("title"),
            "usage_condition": obj.get("usageCondition"),
            "reward": CouponRewardResponse.from_dict(obj.get("reward")) if obj.get("reward") is not None else None,
            "visibility": obj.get("visibility"),
            "timezone": obj.get("timezone"),
            "coupon_id": obj.get("couponId"),
            "created_timestamp": obj.get("createdTimestamp"),
            "status": obj.get("status")
        })
        return _obj




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.


import os
import re
import sys
import subprocess

from setuptools import setup, Command, find_packages
from setuptools.command.test import test as TestCommand

__version__ = ''
with open('linebot/__about__.py', 'r') as fd:
    reg = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
    for line in fd:
        m = reg.match(line)
        if m:
            __version__ = m.group(1)
            break


def _requirements():
    with open('requirements.txt', 'r') as fd:
        return [name.strip() for name in fd.readlines()]


def _requirements_test():
    with open('requirements-test.txt', 'r') as fd:
        return [name.strip() for name in fd.readlines()]


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


class CodegenCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        basedir = os.path.abspath(os.path.dirname(__file__))

        header = (
            "# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
            "#\n"
            "#  *** DO NOT EDIT THIS FILE ***\n"
            "#\n"
            "#  1) Modify linebot/api.py\n"
            "#  2) Run `python setup.py codegen`\n"
            "#\n"
            "# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
            "\n"
        )

        with open(f"{basedir}/linebot/api.py", "r") as original:
            source = original.read()
            import re

            async_source = header + source
            async_source = re.sub("    def (?!__init__)", "    async def ",
                                  async_source)
            async_source = re.sub("from .http_client import HttpClient, RequestsHttpClient", "",
                                  async_source)

            # Change the signature of the __init__.
            # self, channel_access_token
            async_source = re.sub(r"def __init__\(self, channel_access_token,",
                                  "def __init__(self, channel_access_token, async_http_client,",
                                  async_source)
            async_source = re.sub(
                r",\s*timeout=HttpClient.DEFAULT_TIMEOUT, http_client=RequestsHttpClient",
                "", async_source)
            async_source = re.sub(
                r"if http_client:\n"
                + r"\s*self.http_client = http_client\(timeout=timeout\)\n"
                + r"\s*else:\n"
                + r"\s*self.http_client = RequestsHttpClient\(timeout=timeout\)",
                "self.async_http_client = async_http_client", async_source)
            async_source = re.sub(
                r"\"\"\"__init__ method.*?\"\"\"\n",
                '"""__init__ method.' + "\n\n"
                + "    :param str channel_access_token: Your channel access token\n"
                + "    :param str endpoint: (optional) Default is https://api.line.me\n"
                + "    :param str data_endpoint: (optional) Default is https://api-data.line.me\n"
                + "\n\"\"\"\n"
                , async_source, flags=re.DOTALL)
            async_source = re.sub("'line-bot-sdk-python/'", '"line-bot-sdk-python-async/"',
                                  async_source)

            async_source = re.sub('"""linebot.api module."""', '"""linebot.async_api module."""',
                                  async_source)
            async_source = re.sub(
                "self.(_get|_post|_delete|_put)", "await self.\\1", async_source
            )
            async_source = re.sub(
                "self.http_client.(get|post|delete|put)", "await self.async_http_client.\\1",
                async_source
            )
            async_source = re.sub(
                "response.json", "(await response.json)", async_source
            )
            async_source = re.sub(
                "from .http_client import HttpClient, RequestsHttpClient",
                "from .async_http_client import AsyncHttpClient, AiohttpAsyncHttpClient",
                async_source
            )
            async_source = re.sub(
                "linebot.http_client.RequestsHttpClient",
                "linebot.async_http_client.AiohttpAsyncHttpClient",
                async_source
            )
            async_source = re.sub(
                "HttpClient.DEFAULT_TIMEOUT", "AsyncHttpClient.DEFAULT_TIMEOUT", async_source
            )
            async_source = re.sub(
                "RequestsHttpClient", "AiohttpAsyncHttpClient", async_source
            )
            async_source = re.sub(
                "Default is self.http_client.timeout", "Default is self.async_http_client.timeout",
                async_source
            )
            async_source = re.sub(
                "self.__check_error", "await self.__check_error", async_source
            )
            async_source = re.sub(
                "class LineBotApi",
                "class AsyncLineBotApi", async_source
            )
            async_source = re.sub("stream=(stream|False|True), ", "", async_source)

            async_source = re.sub("MessagingApiBlob'", "AsyncMessagingApiBlob'", async_source)
            async_source = re.sub("MessagingApiBlob\\(", "AsyncMessagingApiBlob(", async_source)
            async_source = re.sub("MessagingApi'", "AsyncMessagingApi'", async_source)
            async_source = re.sub("MessagingApi\\(", "AsyncMessagingApi(", async_source)
            async_source = re.sub("ManageAudience'", "AsyncManageAudience'", async_source)
            async_source = re.sub("ManageAudience\\(", "AsyncManageAudience(", async_source)
            async_source = re.sub("Insight'", "AsyncInsight'", async_source)
            async_source = re.sub("Insight\\(", "AsyncInsight(", async_source)
            async_source = re.sub("ChannelAccessToken'", "AsyncChannelAccessToken'", async_source)
            async_source = re.sub("ChannelAccessToken\\(", "AsyncChannelAccessToken(", async_source)

            with open(f"{basedir}/linebot/async_api.py", "w") as output:
                output.write(async_source)

            subprocess.check_call(
                [sys.executable, "-m", "black", f"{basedir}/linebot/async_api.py"],
            )


with open('README.rst', 'r') as fd:
    long_description = fd.read()

setup(
    name="line-bot-sdk",
    version=__version__,
    author="RyosukeHasebe",
    author_email="hsb.1014@gmail.com",
    maintainer="RyosukeHasebe",
    maintainer_email="hsb.1014@gmail.com",
    url="https://github.com/line/line-bot-sdk-python",
    description="LINE Messaging API SDK for Python",
    long_description=long_description,
    license='Apache License 2.0',
    packages=find_packages(include=["linebot*"]),
    python_requires=">=3.9.0",
    install_requires=_requirements(),
    tests_require=_requirements_test(),
    cmdclass={
        'test': PyTest,
        'codegen': CodegenCommand
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Software Development"
    ]
)



# coding: utf-8

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, validator

class GetNumberOfMessageDeliveriesResponse(BaseModel):
    """
    Get number of message deliveries
    https://developers.line.biz/en/reference/messaging-api/#get-number-of-delivery-messages
    """
    status: Optional[StrictStr] = Field(None, description="Status of the counting process.")
    broadcast: Optional[StrictInt] = Field(None, description="Number of messages sent to all of this LINE Official Account's friends (broadcast messages).")
    targeting: Optional[StrictInt] = Field(None, description="Number of messages sent to some of this LINE Official Account's friends, based on specific attributes (targeted messages).")
    auto_response: Optional[StrictInt] = Field(None, alias="autoResponse", description="Number of auto-response messages sent.")
    welcome_response: Optional[StrictInt] = Field(None, alias="welcomeResponse", description="Number of greeting messages sent.")
    chat: Optional[StrictInt] = Field(None, description="Number of messages sent from LINE Official Account Manager [Chat screen](https://www.linebiz.com/jp/manual/OfficialAccountManager/chats/) (only available in Japanese).")
    api_broadcast: Optional[StrictInt] = Field(None, alias="apiBroadcast", description="Number of broadcast messages sent with the `Send broadcast message` Messaging API operation.")
    api_push: Optional[StrictInt] = Field(None, alias="apiPush", description="Number of push messages sent with the `Send push message` Messaging API operation.")
    api_multicast: Optional[StrictInt] = Field(None, alias="apiMulticast", description="Number of multicast messages sent with the `Send multicast message` Messaging API operation.")
    api_narrowcast: Optional[StrictInt] = Field(None, alias="apiNarrowcast", description="Number of narrowcast messages sent with the `Send narrowcast message` Messaging API operation.")
    api_reply: Optional[StrictInt] = Field(None, alias="apiReply", description="Number of replies sent with the `Send reply message` Messaging API operation.")

    __properties = ["status", "broadcast", "targeting", "autoResponse", "welcomeResponse", "chat", "apiBroadcast", "apiPush", "apiMulticast", "apiNarrowcast", "apiReply"]

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('ready', 'unready', 'out_of_service'):
            raise ValueError("must be one of enum values ('ready', 'unready', 'out_of_service')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetNumberOfMessageDeliveriesResponse:
        """Create an instance of GetNumberOfMessageDeliveriesResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetNumberOfMessageDeliveriesResponse:
        """Create an instance of GetNumberOfMessageDeliveriesResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetNumberOfMessageDeliveriesResponse.parse_obj(obj)

        _obj = GetNumberOfMessageDeliveriesResponse.parse_obj({
            "status": obj.get("status"),
            "broadcast": obj.get("broadcast"),
            "targeting": obj.get("targeting"),
            "auto_response": obj.get("autoResponse"),
            "welcome_response": obj.get("welcomeResponse"),
            "chat": obj.get("chat"),
            "api_broadcast": obj.get("apiBroadcast"),
            "api_push": obj.get("apiPush"),
            "api_multicast": obj.get("apiMulticast"),
            "api_narrowcast": obj.get("apiNarrowcast"),
            "api_reply": obj.get("apiReply")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr

class GetStatisticsPerUnitResponseClick(BaseModel):
    """
    GetStatisticsPerUnitResponseClick
    https://developers.line.biz/en/reference/messaging-api/#get-statistics-per-unit-response
    """
    seq: StrictInt = Field(..., description="The URL's serial number.")
    url: StrictStr = Field(..., description="URL.")
    click: Optional[StrictInt] = Field(None, description="Number of times the URL in the bubble was opened.")
    unique_click: Optional[StrictInt] = Field(None, alias="uniqueClick", description="Number of users that opened the URL in the bubble.")
    unique_click_of_request: Optional[StrictInt] = Field(None, alias="uniqueClickOfRequest", description="Number of users who opened this url through any link in the message. If another message bubble contains the same URL and a user opens both links, it's counted only once. ")

    __properties = ["seq", "url", "click", "uniqueClick", "uniqueClickOfRequest"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetStatisticsPerUnitResponseClick:
        """Create an instance of GetStatisticsPerUnitResponseClick from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if click (nullable) is None
        # and __fields_set__ contains the field
        if self.click is None and "click" in self.__fields_set__:
            _dict['click'] = None

        # set to None if unique_click (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_click is None and "unique_click" in self.__fields_set__:
            _dict['uniqueClick'] = None

        # set to None if unique_click_of_request (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_click_of_request is None and "unique_click_of_request" in self.__fields_set__:
            _dict['uniqueClickOfRequest'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetStatisticsPerUnitResponseClick:
        """Create an instance of GetStatisticsPerUnitResponseClick from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetStatisticsPerUnitResponseClick.parse_obj(obj)

        _obj = GetStatisticsPerUnitResponseClick.parse_obj({
            "seq": obj.get("seq"),
            "url": obj.get("url"),
            "click": obj.get("click"),
            "unique_click": obj.get("uniqueClick"),
            "unique_click_of_request": obj.get("uniqueClickOfRequest")
        })
        return _obj




# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr

class ImageSet(BaseModel):
    """
    ImageSet
    """
    id: StrictStr = Field(..., description="Image set ID. Only included when multiple images are sent simultaneously.")
    index: Optional[StrictInt] = Field(None, description="An index starting from 1, indicating the image number in a set of images sent simultaneously. Only included when multiple images are sent simultaneously. However, it won't be included if the sender is using LINE 11.15 or earlier for Android.")
    total: Optional[StrictInt] = Field(None, description="The total number of images sent simultaneously.")

    __properties = ["id", "index", "total"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ImageSet:
        """Create an instance of ImageSet from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ImageSet:
        """Create an instance of ImageSet from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ImageSet.parse_obj(obj)

        _obj = ImageSet.parse_obj({
            "id": obj.get("id"),
            "index": obj.get("index"),
            "total": obj.get("total")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import Field, StrictStr, conlist
from linebot.v3.messaging.models.action import Action
from linebot.v3.messaging.models.template import Template

class ButtonsTemplate(Template):
    """
    ButtonsTemplate
    """
    thumbnail_image_url: Optional[StrictStr] = Field(None, alias="thumbnailImageUrl")
    image_aspect_ratio: Optional[StrictStr] = Field(None, alias="imageAspectRatio")
    image_size: Optional[StrictStr] = Field(None, alias="imageSize")
    image_background_color: Optional[StrictStr] = Field(None, alias="imageBackgroundColor")
    title: Optional[StrictStr] = None
    text: StrictStr = Field(...)
    default_action: Optional[Action] = Field(None, alias="defaultAction")
    actions: conlist(Action) = Field(...)
    type: str = "buttons"

    __properties = ["type", "thumbnailImageUrl", "imageAspectRatio", "imageSize", "imageBackgroundColor", "title", "text", "defaultAction", "actions"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ButtonsTemplate:
        """Create an instance of ButtonsTemplate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of default_action
        if self.default_action:
            _dict['defaultAction'] = self.default_action.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in actions (list)
        _items = []
        if self.actions:
            for _item in self.actions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['actions'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ButtonsTemplate:
        """Create an instance of ButtonsTemplate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ButtonsTemplate.parse_obj(obj)

        _obj = ButtonsTemplate.parse_obj({
            "type": obj.get("type"),
            "thumbnail_image_url": obj.get("thumbnailImageUrl"),
            "image_aspect_ratio": obj.get("imageAspectRatio"),
            "image_size": obj.get("imageSize"),
            "image_background_color": obj.get("imageBackgroundColor"),
            "title": obj.get("title"),
            "text": obj.get("text"),
            "default_action": Action.from_dict(obj.get("defaultAction")) if obj.get("defaultAction") is not None else None,
            "actions": [Action.from_dict(_item) for _item in obj.get("actions")] if obj.get("actions") is not None else None
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
from aiohttp.test_utils import AioHTTPTestCase
from linebot.exceptions import LineBotApiError
from aiohttp import web
from linebot import AsyncLineBotApi
from linebot.aiohttp_async_http_client import AiohttpAsyncHttpClient


class MyAppTestCase(AioHTTPTestCase):
    async def get_application(self):
        self.exception = {
            "message": "user id not found",
            "details": [
                {
                    "message": "user id not found",
                    "property": "... some property"
                }
            ]
        }

        async def profile(request):
            return web.json_response(self.exception, status=404)

        app = web.Application()
        app.router.add_get('//v2/bot/profile/failed', profile)
        return app

    async def test_async_get_profile_exception(self):
        self.server.skip_url_asserts = True
        async_client = AiohttpAsyncHttpClient(session=self.client)
        bot = AsyncLineBotApi('TOKENTOKEN', async_client, endpoint='/')
        with self.assertRaises(LineBotApiError):
            await bot.get_profile('failed')



# flake8: noqa

# import apis into api package
from linebot.v3.messaging.api.messaging_api import MessagingApi
from linebot.v3.messaging.api.messaging_api_blob import MessagingApiBlob


# Async version
from linebot.v3.messaging.api.async_messaging_api import AsyncMessagingApi
from linebot.v3.messaging.api.async_messaging_api_blob import AsyncMessagingApiBlob




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class TemplateImageSize(str, Enum):
    """
    Size of the image.  This is only for the `imageSize` in ButtonsTemplate. Specify one of the following values:  `cover`: The image fills the entire image area. Parts of the image that do not fit in the area are not displayed. `contain`: The entire image is displayed in the image area. A background is displayed in the unused areas to the left and right of vertical images and in the areas above and below horizontal images. 
    """

    """
    allowed enum values
    """
    COVER = 'cover'
    CONTAIN = 'contain'

    @classmethod
    def from_json(cls, json_str: str) -> TemplateImageSize:
        """Create an instance of TemplateImageSize from a JSON string"""
        return TemplateImageSize(json.loads(json_str))





# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class FlexBoxBorderWidth(str, Enum):
    """
    Width of box border. This is only for `borderWidth` in FlexBox. A value of none means that borders are not rendered; the other values are listed in order of increasing width. 
    """

    """
    allowed enum values
    """
    NONE = 'none'
    LIGHT = 'light'
    NORMAL = 'normal'
    MEDIUM = 'medium'
    SEMI_MINUS_BOLD = 'semi-bold'
    BOLD = 'bold'

    @classmethod
    def from_json(cls, json_str: str) -> FlexBoxBorderWidth:
        """Create an instance of FlexBoxBorderWidth from a JSON string"""
        return FlexBoxBorderWidth(json.loads(json_str))





# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import Field, StrictStr
from linebot.v3.messaging.models.message import Message
from linebot.v3.messaging.models.quick_reply import QuickReply
from linebot.v3.messaging.models.sender import Sender

class ImageMessage(Message):
    """
    ImageMessage
    https://developers.line.biz/en/reference/messaging-api/#image-message
    """
    original_content_url: StrictStr = Field(..., alias="originalContentUrl")
    preview_image_url: StrictStr = Field(..., alias="previewImageUrl")
    type: str = "image"

    __properties = ["type", "quickReply", "sender", "originalContentUrl", "previewImageUrl"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ImageMessage:
        """Create an instance of ImageMessage from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of quick_reply
        if self.quick_reply:
            _dict['quickReply'] = self.quick_reply.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of sender
        if self.sender:
            _dict['sender'] = self.sender.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ImageMessage:
        """Create an instance of ImageMessage from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ImageMessage.parse_obj(obj)

        _obj = ImageMessage.parse_obj({
            "type": obj.get("type"),
            "quick_reply": QuickReply.from_dict(obj.get("quickReply")) if obj.get("quickReply") is not None else None,
            "sender": Sender.from_dict(obj.get("sender")) if obj.get("sender") is not None else None,
            "original_content_url": obj.get("originalContentUrl"),
            "preview_image_url": obj.get("previewImageUrl")
        })
        return _obj

